/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.1.0-rc4-master-c26842a
 */
/*  Only used with Theme processes */
html.md-THEME_NAME-theme, body.md-THEME_NAME-theme {
  color: '{{foreground-1}}';
  background-color: '{{background-color}}'; }

md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) {
  background-color: '{{primary-color}}';
  color: '{{primary-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) md-icon {
    color: '{{primary-contrast}}';
    fill: '{{primary-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) .md-button[disabled] md-icon {
    color: '{{primary-contrast-0.26}}';
    fill: '{{primary-contrast-0.26}}'; }
  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-accent {
    background-color: '{{accent-color}}';
    color: '{{accent-contrast}}'; }
    md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-accent .md-ink-ripple {
      color: '{{accent-contrast}}'; }
    md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-accent md-icon {
      color: '{{accent-contrast}}';
      fill: '{{accent-contrast}}'; }
    md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-accent .md-button[disabled] md-icon {
      color: '{{accent-contrast-0.26}}';
      fill: '{{accent-contrast-0.26}}'; }
  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-warn {
    background-color: '{{warn-color}}';
    color: '{{warn-contrast}}'; }
