/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.1.0-rc4-master-c26842a
 */
/*  Only used with Theme processes */
html.md-THEME_NAME-theme, body.md-THEME_NAME-theme {
  color: '{{foreground-1}}';
  background-color: '{{background-color}}'; }

md-slider.md-THEME_NAME-theme ._md-track {
  background-color: '{{foreground-3}}'; }

md-slider.md-THEME_NAME-theme ._md-track-ticks {
  color: '{{background-contrast}}'; }

md-slider.md-THEME_NAME-theme ._md-focus-ring {
  background-color: '{{accent-A200-0.2}}'; }

md-slider.md-THEME_NAME-theme ._md-disabled-thumb {
  border-color: '{{background-color}}';
  background-color: '{{background-color}}'; }

md-slider.md-THEME_NAME-theme._md-min ._md-thumb:after {
  background-color: '{{background-color}}';
  border-color: '{{foreground-3}}'; }

md-slider.md-THEME_NAME-theme._md-min ._md-focus-ring {
  background-color: '{{foreground-3-0.38}}'; }

md-slider.md-THEME_NAME-theme._md-min[md-discrete] ._md-thumb:after {
  background-color: '{{background-contrast}}';
  border-color: transparent; }

md-slider.md-THEME_NAME-theme._md-min[md-discrete] ._md-sign {
  background-color: '{{background-400}}'; }
  md-slider.md-THEME_NAME-theme._md-min[md-discrete] ._md-sign:after {
    border-top-color: '{{background-400}}'; }

md-slider.md-THEME_NAME-theme._md-min[md-discrete][md-vertical] ._md-sign:after {
  border-top-color: transparent;
  border-left-color: '{{background-400}}'; }

md-slider.md-THEME_NAME-theme ._md-track._md-track-fill {
  background-color: '{{accent-color}}'; }

md-slider.md-THEME_NAME-theme ._md-thumb:after {
  border-color: '{{accent-color}}';
  background-color: '{{accent-color}}'; }

md-slider.md-THEME_NAME-theme ._md-sign {
  background-color: '{{accent-color}}'; }
  md-slider.md-THEME_NAME-theme ._md-sign:after {
    border-top-color: '{{accent-color}}'; }

md-slider.md-THEME_NAME-theme[md-vertical] ._md-sign:after {
  border-top-color: transparent;
  border-left-color: '{{accent-color}}'; }

md-slider.md-THEME_NAME-theme ._md-thumb-text {
  color: '{{accent-contrast}}'; }

md-slider.md-THEME_NAME-theme.md-warn ._md-focus-ring {
  background-color: '{{warn-200-0.38}}'; }

md-slider.md-THEME_NAME-theme.md-warn ._md-track._md-track-fill {
  background-color: '{{warn-color}}'; }

md-slider.md-THEME_NAME-theme.md-warn ._md-thumb:after {
  border-color: '{{warn-color}}';
  background-color: '{{warn-color}}'; }

md-slider.md-THEME_NAME-theme.md-warn ._md-sign {
  background-color: '{{warn-color}}'; }
  md-slider.md-THEME_NAME-theme.md-warn ._md-sign:after {
    border-top-color: '{{warn-color}}'; }

md-slider.md-THEME_NAME-theme.md-warn[md-vertical] ._md-sign:after {
  border-top-color: transparent;
  border-left-color: '{{warn-color}}'; }

md-slider.md-THEME_NAME-theme.md-warn ._md-thumb-text {
  color: '{{warn-contrast}}'; }

md-slider.md-THEME_NAME-theme.md-primary ._md-focus-ring {
  background-color: '{{primary-200-0.38}}'; }

md-slider.md-THEME_NAME-theme.md-primary ._md-track._md-track-fill {
  background-color: '{{primary-color}}'; }

md-slider.md-THEME_NAME-theme.md-primary ._md-thumb:after {
  border-color: '{{primary-color}}';
  background-color: '{{primary-color}}'; }

md-slider.md-THEME_NAME-theme.md-primary ._md-sign {
  background-color: '{{primary-color}}'; }
  md-slider.md-THEME_NAME-theme.md-primary ._md-sign:after {
    border-top-color: '{{primary-color}}'; }

md-slider.md-THEME_NAME-theme.md-primary[md-vertical] ._md-sign:after {
  border-top-color: transparent;
  border-left-color: '{{primary-color}}'; }

md-slider.md-THEME_NAME-theme.md-primary ._md-thumb-text {
  color: '{{primary-contrast}}'; }

md-slider.md-THEME_NAME-theme[disabled] ._md-thumb:after {
  border-color: transparent; }

md-slider.md-THEME_NAME-theme[disabled]:not(._md-min) ._md-thumb:after, md-slider.md-THEME_NAME-theme[disabled][md-discrete] ._md-thumb:after {
  background-color: '{{foreground-3}}';
  border-color: transparent; }

md-slider.md-THEME_NAME-theme[disabled][readonly] ._md-sign {
  background-color: '{{background-400}}'; }
  md-slider.md-THEME_NAME-theme[disabled][readonly] ._md-sign:after {
    border-top-color: '{{background-400}}'; }

md-slider.md-THEME_NAME-theme[disabled][readonly][md-vertical] ._md-sign:after {
  border-top-color: transparent;
  border-left-color: '{{background-400}}'; }

md-slider.md-THEME_NAME-theme[disabled][readonly] ._md-disabled-thumb {
  border-color: transparent;
  background-color: transparent; }

md-slider-container[disabled] > *:first-child:not(md-slider),
md-slider-container[disabled] > *:last-child:not(md-slider) {
  color: '{{foreground-3}}'; }
