/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.1.0-rc4-master-c26842a
 */
/*  Only used with Theme processes */
html.md-THEME_NAME-theme, body.md-THEME_NAME-theme {
  color: '{{foreground-1}}';
  background-color: '{{background-color}}'; }

md-switch.md-THEME_NAME-theme .md-ink-ripple {
  color: '{{background-500}}'; }

md-switch.md-THEME_NAME-theme ._md-thumb {
  background-color: '{{background-50}}'; }

md-switch.md-THEME_NAME-theme ._md-bar {
  background-color: '{{background-500}}'; }

md-switch.md-THEME_NAME-theme.md-checked .md-ink-ripple {
  color: '{{accent-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked ._md-thumb {
  background-color: '{{accent-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked ._md-bar {
  background-color: '{{accent-color-0.5}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-focused ._md-thumb:before {
  background-color: '{{accent-color-0.26}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-ink-ripple {
  color: '{{primary-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-primary ._md-thumb {
  background-color: '{{primary-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-primary ._md-bar {
  background-color: '{{primary-color-0.5}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-primary.md-focused ._md-thumb:before {
  background-color: '{{primary-color-0.26}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-ink-ripple {
  color: '{{warn-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-warn ._md-thumb {
  background-color: '{{warn-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-warn ._md-bar {
  background-color: '{{warn-color-0.5}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-warn.md-focused ._md-thumb:before {
  background-color: '{{warn-color-0.26}}'; }

md-switch.md-THEME_NAME-theme[disabled] ._md-thumb {
  background-color: '{{background-400}}'; }

md-switch.md-THEME_NAME-theme[disabled] ._md-bar {
  background-color: '{{foreground-4}}'; }
