/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.9
 */
/*  Only used with Theme processes */
html.md-THEME_NAME-theme, body.md-THEME_NAME-theme {
  color: '{{foreground-1}}';
  background-color: '{{background-color}}'; }

md-checkbox.md-THEME_NAME-theme .md-ripple {
  color: '{{accent-600}}'; }

md-checkbox.md-THEME_NAME-theme.md-checked .md-ripple {
  color: '{{background-600}}'; }

md-checkbox.md-THEME_NAME-theme.md-checked.md-focused .md-container:before {
  background-color: '{{accent-color-0.26}}'; }

md-checkbox.md-THEME_NAME-theme .md-ink-ripple {
  color: '{{foreground-2}}'; }

md-checkbox.md-THEME_NAME-theme.md-checked .md-ink-ripple {
  color: '{{accent-color-0.87}}'; }

md-checkbox.md-THEME_NAME-theme .md-icon {
  border-color: '{{foreground-2}}'; }

md-checkbox.md-THEME_NAME-theme.md-checked .md-icon {
  background-color: '{{accent-color-0.87}}'; }

md-checkbox.md-THEME_NAME-theme.md-checked .md-icon:after {
  border-color: '{{accent-contrast-0.87}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-ripple {
  color: '{{primary-600}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-ripple {
  color: '{{background-600}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-ink-ripple {
  color: '{{foreground-2}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-ink-ripple {
  color: '{{primary-color-0.87}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-icon {
  border-color: '{{foreground-2}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-icon {
  background-color: '{{primary-color-0.87}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked.md-focused .md-container:before {
  background-color: '{{primary-color-0.26}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-icon:after {
  border-color: '{{primary-contrast-0.87}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-indeterminate[disabled] .md-container {
  color: '{{foreground-3}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn .md-ripple {
  color: '{{warn-600}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn .md-ink-ripple {
  color: '{{foreground-2}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-ink-ripple {
  color: '{{warn-color-0.87}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn .md-icon {
  border-color: '{{foreground-2}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-icon {
  background-color: '{{warn-color-0.87}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked.md-focused:not([disabled]) .md-container:before {
  background-color: '{{warn-color-0.26}}'; }

md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-icon:after {
  border-color: '{{background-200}}'; }

md-checkbox.md-THEME_NAME-theme[disabled] .md-icon {
  border-color: '{{foreground-3}}'; }

md-checkbox.md-THEME_NAME-theme[disabled].md-checked .md-icon {
  background-color: '{{foreground-3}}'; }

md-checkbox.md-THEME_NAME-theme[disabled].md-checked .md-icon:after {
  border-color: '{{background-200}}'; }

md-checkbox.md-THEME_NAME-theme[disabled] .md-label {
  color: '{{foreground-3}}'; }
