/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.9
 */
/*  Only used with Theme processes */
html.md-THEME_NAME-theme, body.md-THEME_NAME-theme {
  color: '{{foreground-1}}';
  background-color: '{{background-color}}'; }

md-divider {
  display: block;
  border-top-width: 1px;
  border-top-style: solid;
  margin: 0; }
  md-divider[md-inset] {
    margin-left: 80px; }

.layout-row > md-divider {
  border-top-width: 0;
  border-right-width: 1px;
  border-right-style: solid; }
