
    // de-DE (German)
    module.exports = {
        "am" : "",
        "pm" : "",

        "x": "%d/%m/%y",
        "X": "%H:%M:%S",
        "c": "%a %d %b %Y %H:%M:%S %Z",

        "months" : [
            "Januar",
            "Februar",
            "März",
            "April",
            "Mai",
            "Juni",
            "Juli",
            "August",
            "September",
            "Oktober",
            "November",
            "Dezember"
        ],

        "months_abbr" : [
            "Jan",
            "Febr",
            "März",
            "Apr",
            "Mai",
            "Juni",
            "Juli",
            "Aug",
            "Sept",
            "Okt",
            "Nov",
            "Dez"
        ],

        "days" : [
            "Sonntag",
            "Montag",
            "Dienstag",
            "Mittwoch",
            "Donnerstag",
            "Freitag",
            "Samstag"
        ],

        "days_abbr" : [
            "So",
            "Mo",
            "Di",
            "Mi",
            "Do",
            "Fr",
            "Sa"
        ]
    };

