/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.1.0-rc4-master-c26842a
 */
/*  Only used with Theme processes */
html.md-THEME_NAME-theme, body.md-THEME_NAME-theme {
  color: '{{foreground-1}}';
  background-color: '{{background-color}}'; }

md-toast.md-THEME_NAME-theme .md-toast-content {
  background-color: #323232;
  color: '{{background-50}}'; }
  md-toast.md-THEME_NAME-theme .md-toast-content .md-button {
    color: '{{background-50}}'; }
    md-toast.md-THEME_NAME-theme .md-toast-content .md-button.md-highlight {
      color: '{{accent-color}}'; }
      md-toast.md-THEME_NAME-theme .md-toast-content .md-button.md-highlight.md-primary {
        color: '{{primary-color}}'; }
      md-toast.md-THEME_NAME-theme .md-toast-content .md-button.md-highlight.md-warn {
        color: '{{warn-color}}'; }
